package cn.ibizlab.api.rest.extensions;

import cn.ibizlab.core.uaa.extensions.service.UserRegisterService;
import cn.ibizlab.core.uaa.service.ISysPSSystemService;
import cn.ibizlab.util.domain.IBZUSER;
import cn.ibizlab.util.errors.BadRequestAlertException;
import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.security.AuthTokenUtil;
import cn.ibizlab.util.security.AuthenticationUser;
import cn.ibizlab.util.service.AuthenticationUserService;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;


@RestController
public class UserRegisterResource {

    @Autowired
    private UserRegisterService userRegisterService;
    @Autowired
    private ISysPSSystemService iSysPSSystemService;
    @Autowired
    private AuthTokenUtil jwtTokenUtil;
    @Autowired
    @Qualifier("UAAUserService")
    private AuthenticationUserService userDetailsService;

    @Value("${ibiz.auth.qq.appid:101881963}")
    private String qqAppId;// qq互联appid
    @Value("${ibiz.auth.qq.appkey:}")
    private String qqAppKey;// qq互联appkey
    @Value("${ibiz.auth.qq.redirect_uri:}")
    private String qqRedirectUri;// qq互联应用回调地址

    @Value("${ibiz.auth.wechat.appID:wx4a7473ac9c617f76}")
    private String wechatAppId; // 微信开放平台appid
    @Value("${ibiz.auth.wechat.appsecret:c2b308b9245dffa316b049840fd81a79}")
    private String wechatappsecret; // 微信开放平台appsecret
    @Value("${ibiz.auth.wechat.openid:}")
    private String Avatar;
    @Value("${ibiz.auth.wechat.nickname:}")
    private String Nickname;

    @Value("${ibiz.auth.dingding.appid:dingoatysxi5rpbluqgzpk}")
    private String dingTalkAppId;
    @Value("${ibiz.auth.dingding.appid:z2SuCSciWQJ6VSFX8jvHzTF0G_rEmHpjHrZct0rGnWrLyl8cZiRny72244EvF4SO}")
    private String dingTalkAppSecret;



    public static final String TOKEN = "weixin"; // 这里的token要和微信公众测试号网页填写的token一致

    /**
     * 普通注册
     *
     * @param param
     * @return
     */
    @PostMapping(value = "/uaa/register")
    public ResponseEntity<JSONObject> register(@RequestBody JSONObject param) {
        JSONObject object = new JSONObject();
        // 空校验
        String loginname = param.getString("loginname");
        String personname = param.getString("personname");
        String password = param.getString("password");
        String registerType = param.getString("registerType");
        if (StringUtils.isEmpty(loginname))
            throw new BadRequestAlertException("用户名为空", "UserRegisterResource", "");
        if (StringUtils.isEmpty(personname) && (!"qq".equals(registerType) && !"wechat".equals(registerType)))
            throw new BadRequestAlertException("用户姓名为空", "UserRegisterResource", "");
        if (StringUtils.isEmpty(password))
            throw new BadRequestAlertException("密码为空", "UserRegisterResource", "");
        if (StringUtils.isEmpty(registerType))
            throw new BadRequestAlertException("注册方式为空", "UserRegisterResource", "");

        // 检查用户名是否已被注册
        String sql = "select loginname from ibzuser where loginname=#{et.loginname}";
        Map<String, Object> sqlParam = new HashMap<>();
        sqlParam.put("loginname", loginname);
        List<JSONObject> select = iSysPSSystemService.select(sql, sqlParam);
        if (select.size() > 0)
            throw new BadRequestAlertException("该用户名已被注册", "UserRegisterResource", "");

        // 注册
        IBZUSER ibzuser = new IBZUSER();
        String uuid = UUID.randomUUID().toString();
        ibzuser.setPersonname(personname);
        ibzuser.setPassword(password);
        ibzuser.setLoginname(loginname);
        ibzuser.setUserid("commom-" + uuid);
        userRegisterService.toRegister(ibzuser);

        // 响应结果
        object.put("ibzuser", ibzuser);
        return ResponseEntity.ok().body(object);
    }

    /**
     * 第三方注册：qq、微信
     *
     * @param param
     * @return
     */
    @PostMapping(value = "/uaa/thirdPartRegister")
    public ResponseEntity<JSONObject> thirdPartRegister(@RequestBody JSONObject param) {
        JSONObject object = new JSONObject();
        // 空校验
        String loginname = param.getString("loginname");
        String password = param.getString("password");
        String registerType = param.getString("registerType");
        String code = param.getString("wechatCode");
        System.out.println("code:" + code);
        String state = param.getString("wechatState");
        System.out.println("state:" + state);
        String avatar = param.getString("avatar");
        if (StringUtils.isEmpty(avatar)) {
            avatar = Avatar;
        }
        System.out.println("avatar:" + avatar);
        String nickname = param.getString("nickname");
        if (StringUtils.isEmpty(nickname)) {
            nickname = Nickname;
        }
        System.out.println("nickname:" + nickname);

        if (StringUtils.isEmpty(loginname))
            throw new BadRequestAlertException("用户名为空", "UserRegisterResource", "");
        if (StringUtils.isEmpty(password))
            throw new BadRequestAlertException("密码为空", "UserRegisterResource", "");
        if (StringUtils.isEmpty(registerType))
            throw new BadRequestAlertException("注册方式为空", "UserRegisterResource", "");
        if (StringUtils.isEmpty(code))
            throw new BadRequestAlertException("微信授权code为空", "UserRegisterResource", "");
        if (StringUtils.isEmpty(state))
            throw new BadRequestAlertException("微信授权state为空", "UserRegisterResource", "");

        // 检查用户名是否已被注册
        String sql = "select loginname from ibzuser where loginname=#{et.loginname}";
        Map<String, Object> sqlParam = new HashMap<>();
        sqlParam.put("loginname", loginname);
        List<JSONObject> select = iSysPSSystemService.select(sql, sqlParam);
        if (select.size() > 0)
            throw new BadRequestAlertException("该用户名已被注册", "UserRegisterResource", "");

        IBZUSER ibzuser = new IBZUSER();
        if (StringUtils.isEmpty(avatar)) {
            // 通过code获取微信用户信息
            JSONObject returnObj = userRegisterService.requestWechatUserByCode(code, state ,wechatAppId , wechatappsecret);
            if (!StringUtils.isEmpty(returnObj) && !returnObj.containsKey("errcode")) {
                // 微信注册
                String uuid = UUID.randomUUID().toString();
                ibzuser.setPassword(password);
                ibzuser.setLoginname(loginname);
                ibzuser.setUserid("wechat-" + uuid);
                ibzuser.setPersonname(returnObj.getString("nickname"));
                ibzuser.setAvatar(returnObj.getString("openid"));
                userRegisterService.toRegister(ibzuser);

                object.put("ibzuser", ibzuser);
            }
        } else {
            // 已经有微信用户信息直接注册
            String uuid = UUID.randomUUID().toString();
            ibzuser.setPassword(password);
            ibzuser.setLoginname(loginname);
            ibzuser.setUserid("wechat-" + uuid);
            ibzuser.setPersonname(nickname);
            ibzuser.setAvatar(avatar);
            userRegisterService.toRegister(ibzuser);

            object.put("ibzuser", ibzuser);
        }
        userDetailsService.resetByUsername(ibzuser.getLoginname());
        AuthenticationUser user = userDetailsService.loadUserByLogin(ibzuser.getLoginname(), ibzuser.getPassword());
        final String token = jwtTokenUtil.generateToken(user);
        AuthenticationUser user2 = new AuthenticationUser();
        CachedBeanCopier.copy(user, user2);
        user2.setAuthorities(null);
        user2.setPermissionList(null);
        object.put("token", token);
        object.put("user", user2);


        return ResponseEntity.ok().body(object);
    }

    /**
     * 获取qq互联创建的网站应用appid
     */
    @GetMapping(value = "/uaa/getQQAppId")
    public ResponseEntity<JSONObject> getTecentAppId() {
        JSONObject obj = new JSONObject();
        String appid = qqAppId;
        if (!StringUtils.isEmpty(appid)) {
            obj.put("appid", appid);
        }

        return ResponseEntity.ok(obj);
    }

    /**
     * 获取微信开放平台创建的网站应用appid
     */
    @GetMapping(value = "/uaa/getWechatAppId")
    public ResponseEntity<JSONObject> getWechatAppId() {
        JSONObject obj = new JSONObject();
        String appid = wechatAppId;
        if (!StringUtils.isEmpty(appid)) {
            obj.put("appid", appid);
        }

        return ResponseEntity.ok(obj);
    }

    /**
     * 获取钉钉开放平台创建的网站应用appid
     */
    @GetMapping(value = "/uaa/getDingtalkAppId")
    public ResponseEntity<JSONObject> getDingtalkAppId() {
        JSONObject obj = new JSONObject();
        String appid = dingTalkAppId;
        if (!StringUtils.isEmpty(appid)) {
            obj.put("appid", appid);
        }

        return ResponseEntity.ok(obj);
    }


    /**
     * 根据code查QQ用户
     *
     * @param param
     * @return
     */
    @PostMapping(value = "/uaa/queryQQUserByCode")
    public ResponseEntity<JSONObject> queryQQUserByCode(@RequestBody JSONObject param) {
        JSONObject object = new JSONObject();
        // 空校验
        String code = param.getString("code");
        if (StringUtils.isEmpty(code))
            throw new BadRequestAlertException("code为空", "UserRegisterResource", "");

        // 通过code获取qq用户信息
        String openid = "";
        JSONObject returnObj = userRegisterService.requestQQUserByCode(code, qqRedirectUri, qqAppId, qqAppKey);
        if (!StringUtils.isEmpty(returnObj) && !returnObj.containsKey("errcode")) {
            openid = returnObj.getString("openid");
            String nickname = returnObj.getString("nickname");
            // 将用户信息存到缓存，供当前人后续请求使用
            Avatar = openid;
            Nickname = nickname;
            // 返回用户信息
            object.put("avatar", openid);
            object.put("nickname", nickname);
        }

        //根据openid查用户
        String sql = "select * from ibzuser where avatar=#{et.avatar}";
        Map<String, Object> sqlParam = new HashMap<>();
        sqlParam.put("avatar", openid);
        List<JSONObject> select = iSysPSSystemService.select(sql, sqlParam);
        if (select.size() > 0) {
            IBZUSER ibzuser = select.get(0).toJavaObject(IBZUSER.class);
            object.put("ibzuser", ibzuser);

            userDetailsService.resetByUsername(ibzuser.getLoginname());
            AuthenticationUser user = userDetailsService.loadUserByLogin(ibzuser.getLoginname(), ibzuser.getPassword());
            final String token = jwtTokenUtil.generateToken(user);
            AuthenticationUser user2 = new AuthenticationUser();
            CachedBeanCopier.copy(user, user2);
            user2.setAuthorities(null);
            user2.setPermissionList(null);
            object.put("token", token);
            object.put("user", user2);
        }

        return ResponseEntity.ok().body(object);
    }


    /**
     * 根据code查微信用户
     *
     * @param param
     * @return
     */
    @PostMapping(value = "/uaa/queryWechatUserByCode")
    public ResponseEntity<JSONObject> queryWechatUserByCode(@RequestBody JSONObject param) {
        JSONObject object = new JSONObject();
        // 空校验
        String code = param.getString("code");
        if (StringUtils.isEmpty(code))
            throw new BadRequestAlertException("code为空", "UserRegisterResource", "");

        // 通过code获取微信用户信息
        String openid = "";
        JSONObject returnObj = userRegisterService.requestWechatUserByCode(code, null, wechatAppId, wechatappsecret);
        if (!StringUtils.isEmpty(returnObj) && !returnObj.containsKey("errcode")) {
            openid = returnObj.getString("openid");
            String nickname = returnObj.getString("nickname");
            // 将用户信息存到缓存，供当前人后续请求使用
            Avatar = openid;
            Nickname = nickname;
            // 返回用户信息
            object.put("avatar", openid);
            object.put("nickname", nickname);
        }

        //根据openid查用户
        String sql = "select * from ibzuser where avatar=#{et.avatar}";
        Map<String, Object> sqlParam = new HashMap<>();
        sqlParam.put("avatar", openid);
        List<JSONObject> select = iSysPSSystemService.select(sql, sqlParam);
        if (select.size() > 0) {
            IBZUSER ibzuser = select.get(0).toJavaObject(IBZUSER.class);
            object.put("ibzuser", ibzuser);

            userDetailsService.resetByUsername(ibzuser.getLoginname());
            AuthenticationUser user = userDetailsService.loadUserByLogin(ibzuser.getLoginname(), ibzuser.getPassword());
            final String token = jwtTokenUtil.generateToken(user);
            AuthenticationUser user2 = new AuthenticationUser();
            CachedBeanCopier.copy(user, user2);
            user2.setAuthorities(null);
            user2.setPermissionList(null);
            object.put("token", token);
            object.put("user", user2);
        }

        return ResponseEntity.ok().body(object);
    }


    /**
     * 根据code查钉钉用户
     *
     * @param param
     * @return
     */
    @PostMapping(value = "/uaa/queryDingtalkUserByCode")
    public ResponseEntity<JSONObject> queryDingtalkUserByCode(@RequestBody JSONObject param) {
        JSONObject object = new JSONObject();
        // 空校验
        String code = param.getString("code");
        if (StringUtils.isEmpty(code))
            throw new BadRequestAlertException("code为空", "UserRegisterResource", "");

        // 通过code获取钉钉用户信息
        String openid = "";
        long currentTimeMillis = System.currentTimeMillis();
        JSONObject returnObj = userRegisterService.requestDingtalkUserByCode(code, currentTimeMillis, dingTalkAppId, dingTalkAppSecret);
        if (!StringUtils.isEmpty(returnObj) && !returnObj.containsKey("errcode")) {
            openid = returnObj.getString("openid");
            String nickname = returnObj.getString("nick");
            // 将用户信息存到缓存，供当前人后续请求使用
            Avatar = openid;
            Nickname = nickname;
            // 返回用户信息
            object.put("avatar", openid);
            object.put("nickname", nickname);
        }

        //根据openid查用户
        String sql = "select * from ibzuser where avatar=#{et.avatar}";
        Map<String, Object> sqlParam = new HashMap<>();
        sqlParam.put("avatar", openid);
        List<JSONObject> select = iSysPSSystemService.select(sql, sqlParam);
        if (select.size() > 0) {
            IBZUSER ibzuser = select.get(0).toJavaObject(IBZUSER.class);
            object.put("ibzuser", ibzuser);

            userDetailsService.resetByUsername(ibzuser.getLoginname());
            AuthenticationUser user = userDetailsService.loadUserByLogin(ibzuser.getLoginname(), ibzuser.getPassword());
            final String token = jwtTokenUtil.generateToken(user);
            AuthenticationUser user2 = new AuthenticationUser();
            CachedBeanCopier.copy(user, user2);
            user2.setAuthorities(null);
            user2.setPermissionList(null);
            object.put("token", token);
            object.put("user", user2);
        }

        return ResponseEntity.ok().body(object);
    }


    /**
     * 根据code获取微信用户信息
     *
     * @param param
     * @return
     */
    @PostMapping(value = "/uaa/getWechatUserInfoByCode")
    public ResponseEntity<JSONObject> getWechatUserInfoByCode(@RequestBody JSONObject param) {
        JSONObject object = new JSONObject();
        // 空校验
        String code = param.getString("code");
        if (StringUtils.isEmpty(code))
            throw new BadRequestAlertException("code为空", "UserRegisterResource", "");

        // 通过code获取微信用户信息
        String openid = Avatar;
        String nickname = Nickname;
        object.put("avatar", openid);
        object.put("nickname", nickname);

        return ResponseEntity.ok().body(object);
    }


    /**
     * 根据openId查用户
     *
     * @param param
     * @return
     */
    @PostMapping(value = "/uaa/queryUserByOpenId")
    public ResponseEntity<JSONObject> queryUserByOpenId(@RequestBody JSONObject param) {
        JSONObject object = new JSONObject();
        // 空校验
        String openId = param.getString("openId");
        if (StringUtils.isEmpty(openId))
            throw new BadRequestAlertException("openId为空", "UserRegisterResource", "");

        // 根据openid查用户
        String sql = "select * from ibzuser where avatar=#{et.avatar}";
        Map<String, Object> sqlParam = new HashMap<>();
        sqlParam.put("avatar", openId);
        List<JSONObject> select = iSysPSSystemService.select(sql, sqlParam);
        if (select.size() > 0) {
            // 将查询到的ibzuser对象返回
            IBZUSER ibzuser = select.get(0).toJavaObject(IBZUSER.class);
            object.put("ibzuser", ibzuser);
        }

        return ResponseEntity.ok().body(object);
    }


    /**
     * 响应微信公众测试号发送的Token验证
     *
     * @param signature 　微信加密签名，signature结合了开发者填写的token参数和请求中的timestamp参数、nonce参数
     * @param timestamp 　时间戳
     * @param nonce     　随机数
     * @param echostr   　随机字符串
     * @param response  响应对象
     * @throws IOException
     * @throws NoSuchAlgorithmException
     */
    @GetMapping(value = "/uaa/responseTokenToWeiXin")
    public void responseTokenToWeiXin(String signature, String timestamp, String nonce, String echostr, HttpServletResponse response) throws IOException, NoSuchAlgorithmException {
        // 将token、timestamp、nonce三个参数进行字典序排序
        System.out.println("signature:" + signature);
        System.out.println("timestamp:" + timestamp);
        System.out.println("nonce:" + nonce);
        System.out.println("echostr:" + echostr);
        System.out.println("TOKEN:" + TOKEN);
        String[] params = new String[]{TOKEN, timestamp, nonce};
        Arrays.sort(params);
        // 将三个参数字符串拼接成一个字符串进行sha1加密
        String clearText = params[0] + params[1] + params[2];
        String algorithm = "SHA-1";
        String sign = new String(org.apache.commons.codec.binary.Hex.encodeHex(MessageDigest.getInstance(algorithm).digest((clearText).getBytes()), true));
        // 开发者获得加密后的字符串可与signature对比，标识该请求来源于微信
        if (signature.equals(sign)) {
            // 确认此次GET请求来自微信服务器，原样返回echostr参数内容，则接入生效，成为开发者成功
            response.getWriter().print(echostr);
        }
    }


}
